
# q-expansion of newform 841.2.d.e, downloaded from the LMFDB on 04 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 841
weight = 2
poly_data = [1, -1, 1, -1, 1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[2, [0, -1, 0, 0, 0, 0]]]
aps_data = [[1, -1, 0, 0, 0, 0], [0, 1, 0, 0, -1, 0], [-2, 2, 0, 1, -2, 1], [0, -1, 2, -2, 1, 0], [3, -1, 3, -1, 3, 0], [-1, -3, 1, -3, -1, 0], [2, 0, 2, 0, 0, -2], [1, 1, 1, 0, -1, 1], [0, -2, -1, 2, -1, -2], [0, 0, 0, 0, 0, 0], [2, -2, 0, -1, -4, -1], [-2, -1, 1, 2, 0, 2], [4, 0, 0, -2, 2, 0], [0, 0, 3, -2, 3, 0], [-1, 1, -7, 1, -1, 0], [2, -2, 0, -1, -2, -1], [-10, 0, -2, 0, 0, 2], [2, -3, 4, -4, 3, -2], [-4, -3, 3, 4, 0, -4], [-3, 3, -9, 3, -3, 0], [0, 4, -7, 2, -7, 4], [0, 3, -3, 0, 0, 6], [-5, 5, -5, 0, -1, 1], [6, -6, 0, -5, 6, -5], [3, -3, 3, 0, -5, 5], [0, 4, -9, 8, -9, 4], [-1, -3, -7, -3, -1, 0], [10, -5, 5, -10, 0, -6], [-6, 5, -2, 2, -5, 6], [8, -3, 0, 0, 3, -8], [-5, -1, 7, -1, -5, 0], [0, 4, -5, -4, -5, 4], [14, -5, 5, -14, 0, -2], [0, 10, -1, 6, -1, 10], [-11, 7, -11, 0, -13, 13], [0, -6, 5, 6, 5, -6], [-12, 0, -10, 14, -14, 10], [5, 3, 1, 3, 5, 0], [4, -3, -6, 6, 3, -4], [-12, 0, -2, 12, -12, 2], [0, 0, 0, 3, -2, 3], [-10, 5, -2, 2, -5, 10], [-8, 0, 0, -6, 6, 0], [-7, -7, -7, 0, -7, 7], [0, 12, -3, 10, -3, 12], [-5, 13, -5, 0, -7, 7], [-2, 5, -5, 2, 0, 18], [-2, 15, -15, 2, 0, 6], [10, -10, 0, -3, -4, -3], [6, 3, -4, 4, -3, -6], [2, 0, -8, -2, 2, 8], [22, -17, 16, -16, 17, -22], [-2, 5, -5, 2, 0, -10], [13, -15, 13, 0, 3, -3], [-1, -15, -1, 0, 1, -1], [2, -2, 0, 7, 12, 7], [10, -7, 7, -10, 0, -16], [-1, -1, -1, 0, 9, -9], [-3, 3, 9, 3, -3, 0], [-2, -13, 13, 2, 0, 8], [4, -3, 4, -4, 3, -4], [0, 1, -18, 18, -1, 0], [12, 0, -6, 0, 0, 6], [-16, 3, -3, 16, 0, -6], [0, -14, 9, -18, 9, -14], [-6, 6, 0, -1, 2, -1], [14, 0, -8, -2, 2, 8], [-5, 9, -5, 9, -5, 0], [8, 0, 14, -12, 12, -14], [0, 0, 20, -10, 10, -20], [-4, 4, 0, 9, 0, 9], [9, 3, 7, 3, 9, 0], [-1, 31, -1, 0, 1, -1], [17, -5, 17, 0, -1, 1], [6, -6, 0, -19, 10, -19], [0, -12, 17, 4, 17, -12], [22, 0, 8, -16, 16, -8], [0, 9, -9, 0, 0, 8], [6, -6, 0, -19, 12, -19], [-14, 25, -14, 14, -25, 14], [7, -9, -19, -9, 7, 0], [0, 4, -3, 18, -3, 4], [5, 13, 5, 0, 13, -13], [-10, 10, 0, 13, -22, 13], [3, 9, 3, 0, 3, -3], [12, -5, 5, -12, 0, 6], [0, 6, -1, -18, -1, 6], [-6, -3, 2, -2, 3, 6], [-10, 3, 6, -6, -3, 10], [-6, 0, 4, -14, 14, -4], [14, -25, 18, -18, 25, -14], [0, -16, 21, -14, 21, -16], [0, 8, -11, 0, -11, 8], [18, -18, 0, 1, 14, 1], [0, 16, 7, -12, 7, 16], [-5, 3, -5, 0, -5, 5], [-11, 17, -9, 17, -11, 0], [2, 0, -8, 10, -10, 8], [-6, 0, 10, 8, -8, -10], [-5, -5, -5, 0, 13, -13], [1, -29, 1, 0, 3, -3], [0, 0, -5, -14, -5, 0], [18, 0, 18, 6, -6, -18], [11, -15, -11, -15, 11, 0], [2, -9, 9, -2, 0, -20], [22, -5, 10, -10, 5, -22], [18, -23, 24, -24, 23, -18], [5, -19, 15, -19, 5, 0], [1, -9, 1, 0, -11, 11], [4, -3, 3, -4, 0, 26], [8, -8, 0, 15, 18, 15], [-5, -19, -5, 0, 5, -5], [-10, 3, -3, 10, 0, -20], [-19, -1, -19, 0, 13, -13], [-4, -9, -12, 12, 9, 4], [-16, 11, -30, 30, -11, 16], [-2, 2, 0, -19, 14, -19], [20, -5, 5, -20, 0, 0], [0, 2, -13, -2, -13, 2], [-12, 13, -13, 12, 0, 10], [0, 0, -7, 16, -7, 0], [0, -4, 9, -16, 9, -4], [13, 21, 13, 0, -3, 3], [5, -1, 19, -1, 5, 0], [-20, 20, 0, 19, 0, 19], [26, -26, 0, 1, 14, 1], [7, -13, 29, -13, 7, 0], [0, 10, -15, 38, -15, 10], [-4, 4, 0, 9, 34, 9], [8, -13, 13, -8, 0, -30], [0, -16, -11, 0, -11, -16], [-1, 13, -15, 13, -1, 0], [22, -15, 14, -14, 15, -22], [0, -19, 0, 0, 19, 0], [0, 15, -10, 10, -15, 0], [0, 10, 19, -2, 19, 10], [-7, -9, -7, 0, -3, 3], [-9, 5, -9, 0, -7, 7], [0, -8, -9, 16, -9, -8], [-12, 13, -8, 8, -13, 12], [6, 0, -32, 12, -12, 32], [0, 3, -3, 0, 0, -10], [1, 13, -19, 13, 1, 0], [0, 16, -25, 28, -25, 16], [-16, 0, -22, -4, 4, 22], [-4, 4, 0, -3, -42, -3], [-20, 0, -20, 18, -18, 20], [7, 11, 7, 11, 7, 0], [7, 27, -15, 27, 7, 0], [-30, 7, -10, 10, -7, 30], [-8, -1, -14, 14, 1, 8], [-7, -1, -23, -1, -7, 0], [-3, 7, -25, 7, -3, 0], [18, 0, 30, -6, 6, -30], [-30, 17, -17, 30, 0, 0], [-4, 0, 0, -12, 12, 0], [-8, -1, 1, 8, 0, 8], [-2, 0, 2, -12, 12, -2], [-20, 5, -5, 20, 0, 22], [17, -7, 5, -7, 17, 0], [-19, 17, -19, 0, 5, -5], [-25, 7, -25, 0, -31, 31], [25, -19, 25, 0, 35, -35], [0, 16, 11, 16, 11, 16], [12, -9, 9, -12, 0, -38], [-18, 29, -20, 20, -29, 18], [-4, 4, 0, -17, 12, -17], [-5, 23, 3, 23, -5, 0]]
